module SaveTheMonth
  class ExpectedAmountQuery
    attr_accessor :balance, :amount
    
    def initialize(balance)
      self.balance = balance
    end
    
    def at_beginning
      balance.initial_amount
    end
    
    def today
      at_date Date.new
    end
    
    def at_end
      at_date balance.final_date
    end
    
    def at_date(boundary_date)
      self.amount = balance.initial_amount
      
      (balance.initial_date..boundary_date).each do |date|
        update_with_day(date)
      end
      
      self.amount
    end
    
    private
    
    def update_with_day(date)
      self.amount = balance.movements.inject(self.amount) do |amount, movement|
        if movement.can_perform?(date, balance.initial_date)
          amount + movement.amount
        else
          amount
        end
      end
    end
  end
  
  class QueryDsl
    attr_accessor :balance
    
    def initialize(balance)
      self.balance = balance
    end
    
    def expected_amount
      ExpectedAmountQuery.new(balance)
    end
  end
end